/*******************************************************************************
Module Name: io_control
Module Desc: SPI Serial ROM I/O control logic

Author: Ryan Clarke
Data:   7 August 2015
E-Mail: rfclarke@alum.wpi.edu
*******************************************************************************/

module io_control
    #(
    // SPI data width
    parameter N = 8
    )
    (
    // 50 MHz clock and reset button (active low)
    input wire clk,
    input wire rst_n,
    
    // SPI slave connections
    input wire data_req,
    output wire [N-1:0] data_out,
    output wire data_rdy,
    input wire data_ack,
    input wire cmd_rdy,
    input wire [N-1:0] cmd_in,
    
    // ROM connections
    output wire [3:0] addr,
    input wire [7:0] data
    );
    
    // Command to set ROM read address
    localparam CMD_SET_ADDR = 4'b0001;
    
    wire data_req_edge;
    reg data_req_reg;
    
    reg [3:0] addr_reg, addr_next;
    
    // edge detector for SPI slave data request
    assign data_req_edge = data_req & ~data_req_reg;

    // address and data request registers
    always @(posedge clk, negedge rst_n) begin
        if(~rst_n) begin
            addr_reg <= 0;
            data_req_reg <= 0;
        end
        else begin
            addr_reg <= addr_next;
            data_req_reg <= data_req;
        end
    end
    
    // command decoder logic
    always @* begin
        addr_next = addr_reg;
        
        if(cmd_rdy && (cmd_in[7:4] == CMD_SET_ADDR)) begin            
            addr_next = cmd_in[3:0];
        end
        else if(data_req_edge) begin
            addr_next = addr_reg + 1;
        end
    end
    
    // output logic
    assign addr = addr_reg;
    assign data_out = data;
    assign data_rdy = 1'b1;		// data is always available
    
endmodule
