/*******************************************************************************
 * Module Name: microstick_uart.c
 * Module Desc: Microstick II UART (J6) operations
 * 
 * Author: Ryan Clarke
 * Date:   7 August 2015
 * Email:  rfclarke@alum.wpi.edu
 ******************************************************************************/

#include <xc.h>

#include <ctype.h>
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>

#include "microstick2_uart.h"


static bool uart_buf_rdy = false;
static char uart_buf[UART_BUF_SIZE];


void microstick2_uart_init(unsigned int brg)
{
    // Microstick II UART (J6) Setup
    // - assign U1RX to RP10 (pin 21)
    // - assign U1TX to RP11 (pin 22)
    // - high-speed baud rate generator (BRGH)
    // - 8-bit data, no parity (PDSEL)
    // - 1 stop bit (STSEL)
    // - set baud rate (U1BRG)
    // - disable transmit interrupt (U1TXIE)
    // - enable receive interrupt (U1RXIE)
    // - receive interrupt priority 1 (U1RXIP)
    // - interrupt when any character received (URXISEL)
    // - enable UART (UARTEN)
    // - enable transmitter (UTXEN)
    // - enable receiver (UARTEN)
    RPINR18bits.U1RXR = 10;
    RPOR5bits.RP11R = 3;
    U1MODEbits.BRGH = 1;
    U1MODEbits.PDSEL = 0;
    U1MODEbits.STSEL = 0;
    U1BRG = brg;
    IEC0bits.U1TXIE = 0;
    IEC0bits.U1RXIE = 1;
    IPC2bits.U1RXIP = 1;
    U1STAbits.URXISEL = 0;
    U1MODEbits.UARTEN = 1;
    U1STAbits.UTXEN = 1;
    U1MODEbits.UARTEN = 1;
}

// UART Receive Interrupt Routine
void __attribute__((interrupt, auto_psv)) _U1RXInterrupt(void)
{
    char c;
    static int i = 0;

    while(U1STAbits.URXDA)
    {
        c = U1RXREG;
            
        // a line feed (LF) is the end of an input
        if(c == '\n')
        {
            uart_buf[i] = 0;
            i = 0;
            uart_buf_rdy = true;
        }
        // only copy data if room left in buffer and filter out carriage returns (CR)
        else if((i < UART_BUF_SIZE - 1) && (c != '\r'))
        {
            uart_buf[i] = c;
            i++;
        }
    }
    
    // reset overflow error bit
    if(U1STAbits.OERR)
    {
        U1STAbits.OERR = 0;
    }

    // clear the interrupt flag
    IFS0bits.U1RXIF = 0;
}

void microstick2_uart_putc(char c)
{
    while(U1STAbits.UTXBF);
    
    U1TXREG = c;
}

void microstick2_uart_puts(const char *str)
{
    // output a null terminated string to the UART
    while(*str)
    {
        while(U1STAbits.UTXBF);
        
        U1TXREG = *str;
        str++;
    }
}

const char * microstick2_uart_gets(void)
{
    if(uart_buf_rdy)
    {
        uart_buf_rdy = false;
        return uart_buf;
    }
    else
    {
        return NULL;
    }
}