/*******************************************************************************
 * Module Name: spi_rom.c
 * Module Desc: SPI Serial ROM main program
 * 
 * Author: Ryan Clarke
 * Date:   7 August 2015
 * Email:  rfclarke@alum.wpi.edu
 ******************************************************************************/

#include <xc.h>

#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>

#include "microstick2.h"
#include "microstick2_uart.h"
#include "spi.h"

#include "spi_rom.h"

uint8_t addr = 0;

int main(void)
{
    const char *buf;
    uint8_t data_in;
    command_t cmd;

    // Configure Ports
    // - disable ADC (AD1PCFG)
    // - set LED pin (RA0) as output
    // - turn LED off
    // - set ROM select pin (RA1) as output
    // - deselect ROM (active low)
    AD1PCFG = 0xffff;
    LED_TRIS = 0;
    LED = 0;
    ROM_SELECT_TRIS = 0;
    ROM_SELECT_N = 1;
    
    // Initialize Microstick II UART (J6) for 38400 bps
    microstick2_uart_init(UART_38400);
    
    // Initialize SPI1
    spi1_init();
    
    // Display initial header
    microstick2_uart_puts("SPI ROM Test Commands\r\n");
    microstick2_uart_puts("---------------------\r\n");
    microstick2_uart_puts("d      - dump ROM contents\r\n");
    microstick2_uart_puts("r      - read byte from current ROM address\r\n");
    microstick2_uart_puts("a=addr - set ROM address to <addr> (base 10)\r\n");
    microstick2_uart_puts("---------------------\r\n");
    
    // Display command prompt
    microstick2_uart_puts("> ");
    
    // Main event loop
    while(1)
    {
        // Retrieve a null terminated string from the UART
        buf = microstick2_uart_gets();
        
        if(buf)
        {
            // Parse the string for commands
            cmd = cmd_parser(buf);
            
            switch(cmd)
            {
                case CMD_DUMP:
                    dump_rom();
                    break;
                    
                case CMD_READ:
                    ROM_SELECT_N = 0;
                    data_in = spi1_transfer(SPI_CMD_READ);
                    ROM_SELECT_N = 1;
                    microstick2_uart_putc(data_in);
                    microstick2_uart_puts("\r\n");
                    break;
                    
                case CMD_ADDR:
                    ROM_SELECT_N = 0;
                    spi1_transfer(SPI_CMD_ADDR | addr);
                    ROM_SELECT_N = 1;
                    break;
                    
                case CMD_ERROR:
                    microstick2_uart_puts("invalid command\r\n");
                    break;
                
                case CMD_NOP:                
                default:
                    break;
            }
            
            // Display command prompt
            microstick2_uart_puts("> ");
        }
    }

    return 0;
}

// SPI ROM command parser
command_t cmd_parser(const char *buf)
{
    char hex[3];
    command_t cmd;
    
    // Check the first character of the received buffer for the command
    switch(buf[0])
    {
        case 'd':
            if(buf[1] == 0)
                cmd = CMD_DUMP;
            else
                cmd = CMD_ERROR;
            
            break;
            
        case 'r':
            if(buf[1] == 0)
                cmd = CMD_READ;
            else
                cmd = CMD_ERROR;
            
            break;
            
        case 'a':
            if((buf[1] == '='))
            {
                // grab the address from the stream
                addr = atoi(&buf[2]);
                
                // if the address is greater than 15, set to 15
                if(addr > 0xf)
                    addr = 0xf;
                
                // convert the base-10 address to base-16 and print to the UART
                microstick2_uart_puts("addr: ");
                microstick2_uart_puts(btox(hex, addr));
                microstick2_uart_puts("\r\n");
                cmd = CMD_ADDR;
            }
            else
                cmd = CMD_ERROR;
        
            break;
        
        case 0:
            cmd = CMD_NOP;
            
            break;
        
        default:
            cmd = CMD_ERROR;
    }
    
    return cmd;
}

// Dump ROM contents to UART
void dump_rom(void)
{
    char rom_data[SPI_ROM_SIZE];
    char hex[3];
    int i;

    microstick2_uart_puts("0: ");
    
    // reset the ROM read address to 0
    ROM_SELECT_N = 0;
    spi1_transfer(SPI_CMD_ADDR | 0);
    ROM_SELECT_N = 1;
    
    // copy each ROM byte to a buffer and print the hex to the UART
    for(i = 0; i < SPI_ROM_SIZE; i++)
    {
        ROM_SELECT_N = 0;
        rom_data[i] = spi1_transfer(SPI_CMD_READ);
        ROM_SELECT_N = 1;
        
        microstick2_uart_puts(btox(hex, rom_data[i]));
        microstick2_uart_putc(' ');
    }
    
    microstick2_uart_putc(' ');
    
    // print the ASCII of the ROM data to the UART
    for(i = 0; i < SPI_ROM_SIZE; i++)
    {
        if(isprint(rom_data[i]))
            microstick2_uart_putc(rom_data[i]);
        else
            microstick2_uart_putc('.');
    }

    microstick2_uart_puts("\r\n");
}

// Convert 8-bit unsigned integer to hexadecimal string
char * btox(char *buf, uint8_t val)
{
    int r, q;
    
    buf[2] = 0;
    
    // quotient and remainder
    q = val / 16;
    r = val % 16;
    
    if(r < 10)
        buf[1] = r + '0';
    else
        buf[1] = r - 10 + 'a';
    
    r = q % 16;
    
    if(r < 10)
        buf[0] = r + '0';
    else
        buf[0] = r - 10 + 'a';
    
    return buf;
}